{% load i18n fluent_blogs_tags %}
{% with next_entry=object.next_entry previous_entry=object.previous_entry %}

  <div class="entry-navigation">
    {% if next_entry or previous_entry %}
      <nav class="next-previous-entry">
        {% if previous_entry %}<span class="previous-entry"><a href="{% get_entry_url previous_entry %}" title="{{ previous_entry.title }}" rel="prev"><span class="nav-arrow">&larr; </span>{% trans "Previous" %}</a></span>{% endif %}
        {% if next_entry %}<span class="next-entry"><a href="{% get_entry_url next_entry %}" title="{{ next_entry.title }}" rel="prev">{% trans "Next" %}<span class="nav-arrow"> &rarr;</span></a></span>{% endif %}
      </nav>
    {% endif %}

    {% if object.related_published %}
      <div class="entry-related">
        <h3>{% trans "Related entries" %}</h3>
        <ul>
          {% for entry in object.related_published %}
            <li><a href="{% get_entry_url entry %}" title="{{ entry.title }}" rel="bookmark">{{ entry.title }}</a></li>
          {% endfor %}
        </ul>
      </div>
    {% endif %}
  </div>

{% endwith %}
