{% extends "fluent_blogs/entry_archive.html" %}
{% load i18n %}

{% block meta-description %}{% blocktrans with day=day|date:"DATE_FORMAT" %}Archives of {{ day }}{% endblocktrans %}{% endblock %}

{% block title %}{% blocktrans with day=day|date:"DATE_FORMAT" %}Archives of {{ day }}{% endblocktrans %}{% endblock %}

{% block content_title %}<h1>{% blocktrans with day=day|date:"DATE_FORMAT" %}Archives of {{ day }}{% endblocktrans %}</h1>{% endblock %}
